<?php
namespace Teknisa\EST\Modules\Produto\Services;

use Teknisa\EST\Modules\Produto\Factories\Produto as ProdutoFactory;
use Teknisa\EST\Modules\Produto\Services\ProdutoUtilities;

class Produto {

    protected $produtoUtilities;
    protected $produtoFactory;

    public function __construct(ProdutoFactory $produtoFactory, ProdutoUtilities $produtoUtilities) {
        $this->produtoUtilities = $produtoUtilities;
        $this->produtoFactory = $produtoFactory;
    }

    public function alreadyRelatedToTpTree($cdproduto, $nrtparvprod) {
        $params = array('CDPRODUTO' => $cdproduto, 
                        'NRORG' => $this->produtoUtilities->getNrorg());
        $types = $this->produtoFactory->getTpTreeProduct($params);
        foreach ($types as $type) {
            if ($type['NRTPARVPROD'] == $nrtparvprod) {
                throw new \Exception('O produto já se encontra cadastrado para outro grupo no mesmo tipo de árvore.', 1);
            }
        }
    }

    public function applyRulesToCompraType($row) {
        if (!isset($row['CDPRODESTO'])) {
            throw new \Exception('Erro. Produto de Estoque é obrigatório para um produto de compra.',1);
        } else if ($row['CDPRODUTO'] === $row['CDPRODESTO']) {
            throw new \Exception('Erro. Produto de compra com código do produto iqual ao produto de estoque.',1);
        }

        if (!isset($row['VRFATOCONV'])) {
            throw new \Exception('Erro. Fator de Conversão é obrigatório para um produto de compra.',1);
        }
        
        $row['VRPESOUNID'] = null;
        $row['VRPEFATOCORR'] = null;
        $row['CDPRODPROD'] = null;
        $row['VRFATOCONVPR'] = null;
        $row['IDPRODPROD'] = null;
        return $row;
    }

    public function applyRulesToEstoqueType($row) {
        if (isset($row['CDPRODPROD']) && !isset($row['VRFATOCONVPR'])) {
            throw new \Exception('Erro. É necessário informar o Fator de Conversão Produção quando se seleciona um Produto de Produção.',1);
        }

        $row['VRFATOCONV'] = null;
        $row['CDPRODESTO'] = $row['CDPRODUTO'];
        return $row;
    }

    public function arvprodContainsProdnivante($cdarvprod, $cdprodnivante) {
        if (!$this->isLevelOne($cdarvprod)) {
            $params = array('CDPRODUTO'=>$cdprodnivante,
                            'NRORG'=>$this->produtoUtilities->getNrorg());
            $cdprodnivante = $this->produtoFactory->getArvprodByCdproduto($params);
            if (!isset($cdprodnivante)) {
                throw new \Exception('Erro. Não existe o produto pai informado.', 1);
            } else if (substr($cdarvprod, 0, (strlen($cdprodnivante))) !== $cdprodnivante) {
                throw new \Exception('Erro. O código de Produto não está de acordo com o produto pai selecionado.', 1);
            }
        }
    }

    public function canBeDeletedTree($cdarvprod, $cdproduto) {
        $params = array('CDPRODUTO'=>$cdproduto,
                        'NRORG'=>$this->produtoUtilities->getNrorg());
        $children = $this->produtoFactory->getChildren($params);
        if (count($children) > 0) {
            throw new \Exception('Exclusão não permitida. Existem produtos relacionados.', 1);
        }
        
    }

    public function correctLevelByArvprod($cdarvprod, $cdnvproduto) {
        $levelByCdarvprod = $this->getCdnvprodutoByCdarvprod($cdarvprod);
        if ($levelByCdarvprod != $cdnvproduto) {
            throw new \Exception('Erro. O nível do produto não corresponde ao Código de Árvore.', 1);
        }
    }

    public function existArvprod($cdarvprod) {
        $params = array('CDARVPROD'=>$cdarvprod,
                        'NRORG'=>$this->produtoUtilities->getNrorg());
        $othersProducts = $this->produtoFactory->getProductByArvprod($params);
        
        if ($othersProducts !== false) {
            throw new \Exception('Erro. Código de Produto já cadastrado para outro produto.',1);
        }
    }

    public function existProduct($cdarvprod) {
        $params = array('CDPRODUTO'=>$cdarvprod,
                        'NRORG'=>$this->produtoUtilities->getNrorg());
        $product = $this->produtoFactory->getProductByCode($params);
        
        if ($product) {
            return true;
        } else {
            return false;
        }
    }

    public function getProductMask() {
        $params = array('NRORG'=>$this->produtoUtilities->getNrorg());
        return $this->produtoFactory->getProductMask($params);
    }

    public function getCdnvprodutoByCdarvprod($cdarvprod) {
        $nv = 1;
        $mask = $this->getProductMask();
        $mask = explode('.', $mask);

        for ($i=0; $i < count($mask); $i++) { 
            $sessionLenght = strlen($mask[$i]);
            $arvprodLenght = strlen($cdarvprod);
            if ($sessionLenght === $arvprodLenght) {
                $nv = $nv + $i;
                $i = count($mask);
            }
            $cdarvprod = substr($cdarvprod, ($sessionLenght - $arvprodLenght));
        }

        return $nv;
    }

    public function getNewCdproduto($cdarvprod) {
        $gereratedCode =  $this->produtoUtilities->newCode();
        
        if (strlen($cdarvprod) <= strlen($gereratedCode) && !$this->existProduct($cdarvprod)) {
            return $cdarvprod;
        } else {
            return $gereratedCode;
        }
    }

    public function getSugestedCdarvprod($parentcdproduto, $parentnvproduto) {
        $params = array('CDPRODUTO'=>$parentcdproduto,
                        'NRORG'=>$this->produtoUtilities->getNrorg());
        $children = $this->produtoFactory->getChildren($params);
        $mask = $this->getProductMask();
        $mask = explode('.', $mask);
        $sessionMask = $mask[intval($parentnvproduto)];
        $zeroFill = strlen($sessionMask);
        if (count($children) === 0) {
            $cdarvprod = str_pad('1', $zeroFill, "0", STR_PAD_LEFT);
        } else {
            $brothersArvprod = array_map(function($brother) {
                return $brother['CDARVPROD'];
            }, $children);
            for ($i=0; $i < count($children); $i++) { 
                $sessionBrotherCdarvprod = substr($children[$i]['CDARVPROD'], (strlen($sessionMask) * (-1)));
                $cdarvprod = $this->incrementArvProd($sessionBrotherCdarvprod, $sessionMask);
                if ($this->respectMask($sessionMask, $cdarvprod) && !in_array($cdarvprod, $brothersArvprod)) {
                    $i = count($children);
                }
            }
        }
        
        $parentarvprod = $this->produtoFactory->getArvprodByCdproduto($params);
        $cdarvprod = $parentarvprod.$cdarvprod;
        return $cdarvprod;
    }

    private function isLevelOne($cdarvprod) {
        $level = $this->getCdnvprodutoByCdarvprod($cdarvprod);
        if ($level === 1) {
            return true;
        } else {
            return false;
        }
    }
    
    private function imcrementOne($val, $mask) {
        if ($mask === 'a') {
            if ($val == 9) {
                return 'A';
            } else if ($val === 'Z') {
                return '0';
            } else {
                return ++$val;
            }
        } else {
            $val++;
            if (strlen($val) != strlen($mask)) {
                return '0';
            } else {
                return $val;
            }
        }
    }
    
    private function incrementArvProd($arvprod, $mask) {
        $lastCaracterPostion = strlen($arvprod)- 1;
        $explodedArvprod = str_split($arvprod);
        $explodedMask = str_split($mask);
        
        $explodedArvprod[$lastCaracterPostion] = $this->imcrementOne($explodedArvprod[$lastCaracterPostion], $explodedMask[$lastCaracterPostion]);
        $lastCarcter = $explodedArvprod[$lastCaracterPostion];

        if ($explodedArvprod[$lastCaracterPostion] == 0 && strlen($arvprod) > 1) {
            array_pop($explodedArvprod);
            array_pop($explodedMask);
            $arvprod = implode("", $explodedArvprod);
            $mask = implode("", $explodedMask);
            $arvprod = $this->incrementArvProd($arvprod, $mask).$lastCarcter;
        } else {
            $arvprod = implode("", $explodedArvprod);
        }
        
        return $arvprod;
    }

    public function newProductType($product) {
        if (isset($product['IDESTOCOMP'])) {
            return $product['IDESTOCOMP'];
        } else {
            throw new \Exception('Erro. Não foi informado se o produto é de estoque ou de compra.',1);
        }
    }

    public function productType($product) {
        $params = array(
            'CDPRODUTO'=>$product['CDPRODUTO'],
            'NRORG'=>$this->produtoUtilities->getNrorg()
        );
        $productInDbType = $this->produtoFactory->getProductType($params);
        return $productInDbType;
    }

    public function removeMask($cdarvprodWithMask) {
        $cdarvprod = '';
        $cdarvprodWithMask = explode('.', $cdarvprodWithMask);
        foreach ($cdarvprodWithMask as $value) {
            $cdarvprod = $cdarvprod.$value;
        }

        return strtoupper($cdarvprod);
    }

    private function respectMask($mask, $value) {
        if (strlen($mask) === strlen($value)) {
            return true;
        } else {
            return false;
        }
    }

    public function validateType($cdproduto, $supposedType) {
        $params = array(
            'CDPRODUTO'=>$cdproduto,
            'NRORG'=>$this->produtoUtilities->getNrorg()
        );
        $productInDbType = $this->produtoFactory->getProductType($params);
        if ($productInDbType !== $supposedType) {
            throw new \Exception('Erro. Houve alteração no tipo do protudo, essa edição não é permitida.',1);
        }
    }
    
    public function getCdSegMercFilial($cdFilial) {
        $resultado = $this->produtoFactory->getSegMercFilial($cdFilial);
        if(isset($resultado[0])){
            return $resultado[0]["CDSEGMER"];
        } else{
            return null;  
        }
    }  

}