<?php
/**
 * @todo Substituir valores "hardcoded" por constantes.
 * @todo Implementar coleções.
 * @todo Pensar em uma implementação usando uma instancia de NFE.
 * @todo Adicionar validações, e suas mensagens.
 * @todo Tratar excessões.
 * @todo Refatorar ambiente.
 */

namespace Fiscal\Logic;

use \Geral\Entities\Encargo;
use \Geral\Entities\Filial;
use \Geral\Entities\Imposto;
use \Geral\Entities\Itemencargo;
use \Geral\Entities\Itemproduto;
use \Geral\Entities\Produto;
use \Fiscal\Entities\Imposnftot;
use \Fiscal\Entities\IntgItemencargo;
use \Fiscal\Entities\IntgItemproduto;
use \Fiscal\Entities\IntgNotafiscal;
use \Fiscal\Entities\IntgTrabitproimpos;
use \Fiscal\Entities\Itemproimpos;
use \Fiscal\Entities\Notafiscal;

class NotaFiscalEletronica
{
    static $valorImpostoTotalizado = array();
    static $valorBaseTotalizado = array();

    public static function importaDocumento($nrorg, $cdForneced, $cdSerie, $nrNotaFisc, $idTipoOperacao)
    {
        $nfe = IntgNotafiscal::retornaChaveAcessoDocumento($nrorg, $cdForneced, $cdSerie, $nrNotaFisc);
        if($nfe) {
            $connection = \Setforms2\Runtime\Registry::getEntityManagerConnection();
            $connection->beginTransaction();
            try {
                $notaFiscal = self::importaNotaFiscal($nfe, $idTipoOperacao);
                self::importaProdutos($nfe, $notaFiscal);
                self::importaEncargos($nfe, $notaFiscal);
                self::totalizaImpostos($notaFiscal);
                \Setforms2\Runtime\Registry::getEntityManager()->flush();
                $connection->commit();
                return $notaFiscal;
            } catch (\Exception $e) {
                $connection->rollback();
                throw $e;
            }
        }

        return false;
    }

    protected static function totalizaImpostos($notaFiscal) {
        foreach (self::$valorBaseTotalizado as $codigoImposto => $valorBaseTotalizado) {
            $valorImpostoTotalizado = self::$valorImpostoTotalizado[$codigoImposto];

            $impostoTotal = new Imposnftot();
            $impostoTotal->setLanctonf($notaFiscal);
            $impostoTotal->setImposto(Imposto::getObject($codigoImposto));
            $impostoTotal->setVrtotimponf($valorImpostoTotalizado);
            $impostoTotal->setVrtotbcalcnf($valorBaseTotalizado);
            $impostoTotal->setCdoperinclusao(\Ambiente::getCdOperador());
            $impostoTotal->persist();
        }
    }

    protected static function importaEncargos($nfe, $notaFiscal) {
        $itemEncargosNFE = IntgItemencargo::findByAcessonfe($nfe);
        foreach ($itemEncargosNFE as $itemEncargoNFE) {
            self::importaEncargo($notaFiscal, $itemEncargoNFE);
        }
    }

    protected static function importaEncargo(Notafiscal $notaFiscal, IntgItemencargo $itemEncargoNFE) {
        $itemEncargo = new Itemencargo();
        $itemEncargo->setLanctonf($notaFiscal);
        $itemEncargo->setEncargo($itemEncargoNFE->getEncargo());
        $itemEncargo->setVritemenca($itemEncargoNFE->getVritemenca());
        $itemEncargo->setDtinclusao($itemEncargoNFE->getDtinclusao());
        $itemEncargo->persist();
    }

    protected static function importaProdutos(IntgNotafiscal $nfe, Notafiscal $notaFiscal) {
        $itensNFE = IntgItemproduto::findByAcessonfe($nfe);
        foreach ($itensNFE as $itemNFE) {
            self::importaProduto($notaFiscal, $itemNFE);
        }
    }

    protected static function importaProduto(Notafiscal $notaFiscal, IntgItemproduto $itemNFE)
    {
        $itemProduto = new Itemproduto();
        $itemProduto->setLanctonf($notaFiscal);
        $itemProduto->setNrseqitprod($itemNFE->getNrseqitprod());
        $itemProduto->setProduto($itemNFE->getProduto());
        $itemProduto->setCdprodemis($itemNFE->getCdproduto());
        $itemProduto->setDsprodemis($itemNFE->getNmproduto());
        $itemProduto->setIdatuvrestoq('N');
        $itemProduto->setQtitemprod($itemNFE->getQtitemprod());
        $itemProduto->setVrunititem($itemNFE->getVrunititem());
        $itemProduto->setVrdescitem($itemNFE->getVrdescitem());
        $itemProduto->setVrtotaitem($itemNFE->getVrtotaitem());
        $itemProduto->setVrcontitem($itemNFE->getVrcontitem());
        $itemProduto->setIdconforitem('S');
        $itemProduto->setVrbrutitem($itemNFE->getVrbrutitem());
        $itemProduto->setDtinclusao($itemNFE->getDtinclusao());
        $itemProduto->setLocalestoq($itemNFE->getLocalEstoque());
        $itemProduto->setCdfisopeitem($itemNFE->getCdfisopeitem());
        $itemProduto->setCdsitutribpr($itemNFE->getCdsitutribpr());
        $itemProduto->persist();

        self::importaItemProdImposto($itemProduto, $itemNFE);
    }


    protected static function importaItemProdImposto($itemProduto, $itemNFE) {
        $itemProdutoImpostosNFE = IntgTrabitproimpos::findBySeqimpoit($itemNFE);
        foreach ($itemProdutoImpostosNFE as $itemProdutoImpostoNFE) {
            $itemProdutoImposto = new Itemproimpos();
            $itemProdutoImposto->setSeqitprod($itemProduto);
            $itemProdutoImposto->setImposto(Imposto::getObject($itemProdutoImpostoNFE->getCdimpostoit()));
            $itemProdutoImposto->setVrcontitimp($itemProdutoImpostoNFE->getVrcontitimp());
            $itemProdutoImposto->setVrperredbcit($itemProdutoImpostoNFE->getVrperredbcit());
            $itemProdutoImposto->setVrbasecalcit($itemProdutoImpostoNFE->getVrbasecalcit());
            $itemProdutoImposto->setVraliqitem($itemProdutoImpostoNFE->getVraliqitem());
            $itemProdutoImposto->setVrpardedit('1');
            $itemProdutoImposto->setVrimpostoit($itemProdutoImpostoNFE->getVrimpostoit());
            $itemProdutoImposto->setDtinclusao($itemProdutoImpostoNFE->getDtinclusao());
            $itemProdutoImposto->setCdfiscoperit($itemProdutoImpostoNFE->getCdfiscoperit());
            $itemProdutoImposto->setNritproimpos($itemProdutoImpostoNFE->getNritproimpos());
            $itemProdutoImposto->persist();
            self::somaValorImposto($itemProdutoImposto);
        }
    }

    protected static function somaValorImposto(Itemproimpos $itemProdutoImposto) {
        $incrementaPosicaoArray = function (array &$variavel, $chave, $valor) {
            if(!isset($variavel[$chave])) $variavel[$chave] = 0;
            $variavel[$chave] += $valor;
        };

        $codigoImposto = $itemProdutoImposto->getImposto()->getCdimposto();
        $incrementaPosicaoArray(self::$valorBaseTotalizado, $codigoImposto, $itemProdutoImposto->getVrbasecalcit());
        $incrementaPosicaoArray(self::$valorImpostoTotalizado, $codigoImposto, $itemProdutoImposto->getVrimpostoit());
    }


    /**
     * @static
     * @param IntgNotafiscal $nfe
     * @param $idTipoOperacao
     * @return Notafiscal
     */
    protected static function importaNotaFiscal(IntgNotafiscal $nfe, $idTipoOperacao)
    {
        $nota_fiscal = new Notafiscal();
        $nota_fiscal->setFilial($nfe->getFilial());
        $nota_fiscal->setNrlanctonf($nfe->getNrlanctonf());
        $nota_fiscal->setFilirece($nfe->getFilialRecebimento());
        $nota_fiscal->setEmpresa($nfe->getEmpresa());
        $nota_fiscal->setInscesta($nfe->getInscricaoEstadual());
        $nota_fiscal->setIdtpijurnf($nfe->getIdtpijurnf());
        $nota_fiscal->setNrinsjurnf($nfe->getNrinsjurnf());
        $nota_fiscal->setForneced($nfe->getFornecedor());
        $nota_fiscal->setModdocfisc($nfe->getModeloDocumentoFiscal());
        $nota_fiscal->setIdbloqueado('N');
        $nota_fiscal->setIddocfiscal('S');
        $nota_fiscal->setIddocfinanc('S');
        $nota_fiscal->setNrnotafisc($nfe->getNrnotafisc());
        $nota_fiscal->setDtemissao($nfe->getDtemissao());
        $nota_fiscal->setDtentrsaid($nfe->getDtentrsaid());
        $nota_fiscal->setVrprodutos($nfe->getVrprodutos());
        $nota_fiscal->setVrnotafisc($nfe->getVrnotafisc());
        $nota_fiscal->setEntrsaidop(self::getTipoOperac($idTipoOperacao, $nfe));
        $nota_fiscal->setSerieentr($nfe->getSerieEntrada());
        $nota_fiscal->setIdstatus('A');
        $nota_fiscal->setIdsistema('DFE');
        $nota_fiscal->setIdorigdestnf($nfe->getIdorigdestnf());
        $nota_fiscal->setDtlancamennf($nfe->getDtlancamennf());
        $nota_fiscal->setDtcompfatur(new \DateTime());
        $nota_fiscal->setNracessonfe($nfe->getNracessonfe());
        $nota_fiscal->setIdstatusnfe($nfe->getIdstatusnfe());
        $nota_fiscal->setIdimpressodanfe('N');
        $nota_fiscal->setIdtipoemisnfe($nfe->getIdtipoemisnfe());
        $nota_fiscal->setIdexpnfep('N');
        $nota_fiscal->setIdexpnfesanand('N');
        $nota_fiscal->setIdexpnfemanaus('N');
        $nota_fiscal->setIdemailnfe('P');
        $nota_fiscal->setIdstatusctbnf('P');
        $nota_fiscal->setDtinclusao($nfe->getDtinclusao());
        $nota_fiscal->setIdverifnfconf('N');
        $nota_fiscal->setIdnfdispfin('S');
        $nota_fiscal->setIdtpambnfe('0');
        $nota_fiscal->persist();
        return $nota_fiscal;
    }

    /**
     * @static
     * @param $idTipoOperacao
     * @param \Fiscal\Entities\IntgNotafiscal $nfe
     * @return object
     */
    protected static function getTipoOperac($idTipoOperacao, IntgNotafiscal $nfe)
    {
        $tipooperacId = array(
            "cdtipooper"    => $idTipoOperacao,
            "nrorg"         => $nfe->getNrorg(),
            "identrsaidop"  => \Geral\Entities\Tipooperac::OPERACAO_ENTRADA,
        );

        return \Geral\Entities\Tipooperac::getObject($tipooperacId);
    }

}
